var conf = require('./conf.js');
var async = require('async');
var mkdirp = require('mkdirp');
var rimraf = require('rimraf');
var path = require('path');

async.series([
  setup,
  download,
  chmodChromeDr.bind(null, conf.chromeDriver.path)
], end);

function setup(cb) {
  async.series([
    rimraf.bind(null, conf.selenium.path),
    rimraf.bind(null, path.dirname(conf.chromeDriver.path)),
    mkdirp.bind(null, path.dirname(conf.selenium.path)),
    mkdirp.bind(null, path.dirname(conf.chromeDriver.path))
  ], cb);
}

function download(cb) {
  async.parallel([
    installChromeDr.bind(null, conf.chromeDriver.path, conf.chromeDriver.version),
    installSelenium.bind(null, conf.selenium.path, conf.selenium.version)
  ], cb)
}

function end(err) {
  console.log('Installation finished');
  if (err) {
    throw err
  }
}

function installSelenium(to, version, cb) {
  var seleniumStandaloneUrl =
    'http://selenium-release.storage.googleapis.com/%s/selenium-server-standalone-%s.jar';

  var url = require('util').format(seleniumStandaloneUrl,
    version.slice(0, version.lastIndexOf('.')),
    version);

  getDownloadStream(url, function(err, stream) {
    if (err) {
      return cb(err);
    }

    stream
      .pipe(require('fs').createWriteStream(to))
      .once('error', cb.bind(null, new Error('Could not write to ' + to)))
      .once('finish', cb);
  });
}

function chmodChromeDr(where, cb) {
  console.log('chmod+x chromedriver');
  require('fs').chmod(where, 0755, cb);
}

function installChromeDr(to, version, cb) {
  var path = require('path');
  var util = require('util');

  var chromedriverUrl = 'http://chromedriver.storage.googleapis.com/%s/chromedriver_%s.zip';
  var platform = getChromeDriverPlatform();

  if(platform instanceof Error) {
    return cb(platform);
  }

  var downloadUrl = util.format(chromedriverUrl, version, platform);

  getDownloadStream(downloadUrl, function(err, stream) {
    if (err) {
      return cb(err);
    }

    var unzip = require('unzip');

    console.log('Unzipping ' + downloadUrl);

    stream
      .pipe(require('unzip').Parse())
      .once('entry', function(file) {
        file
          .pipe(require('fs').createWriteStream(to))
          .once('error', cb.bind(null, new Error('Could not write to ' + to)))
          .once('finish', cb)
      })
      .once('error', cb.bind(null, new Error('Could not unzip ' + downloadUrl)))
  })
}

function getDownloadStream(downloadUrl, cb) {
  var proxy = process.env.HTTP_PROXY || process.env.http_proxy;

  var requestOpts = downloadUrl;

  if (proxy) {
    var regexp = /(https?:\/\/)?([^:/]*)/;
    requestOpts = {
      host: proxy.match(regexp)[2],
      port: proxy.match(/:(\d+)/)[1] || 8080,
      path: downloadUrl,
      headers: {
        Host: downloadUrl.match(regexp)[2]
      }
    };
  }

  var r =
    require('http')
      .request(requestOpts, function(res) {
        console.log('Downloading ' + downloadUrl, res.statusCode);

        if (res.statusCode === 302 && res.headers.location) {
          r.abort();
          return getDownloadStream(res.headers.location, cb);
        }

        if (res.statusCode !== 200) {
          return cb(new Error('Could not download ' + downloadUrl));
        }

        cb(null, res);
      })
      .once('error', cb.bind(null, new Error('Could not download ' + downloadUrl)))

  // initiate request
  r.end();
}

function getChromeDriverPlatform() {
  var platform;

  if (process.platform === 'linux') {
    platform = 'linux' + ( process.arch === 'x64' ? '64' : '32' );
  } else if (process.platform === 'darwin') {
    platform = 'mac32';
  } else if (process.platform === 'win32') {
    platform = 'win32'
  } else {
    return new Error('Platform not supported');
  }

  return platform;
}